import { ImageLike, Page } from 'tesseract.js';
declare type ProgressListener = (event: {
    status: string;
}) => void;
declare type AppendHtmlOptions = {
    adjustFontSize?: boolean;
    scale?: number;
};
declare type Options = {
    language?: string;
    lineClassName?: string;
    wordClassName?: string;
};
export default class Ocr {
    data: Page;
    language: string;
    lineClassName: string;
    wordClassName: string;
    progressListeners: ProgressListener[];
    minFontSize: number;
    constructor({ language, lineClassName, wordClassName }?: Options);
    addProgressListener(listener: ProgressListener): void;
    removeProgressListener(listener: ProgressListener): void;
    /**
     * Parse image with tesseract
     *
     * @param {ImageLike} image The image instance.
     */
    parseImage(image: ImageLike): Promise<void>;
    private updateTextSize;
    private updatePosition;
    private mapWords;
    private mapLine;
    /**
     * Append generated html to the container.
     *
     * @param {HTMLElement} container The container where elements should append.
     * @param {AppendHtmlOptions} options The options for customization.
     */
    appendHtml(container: HTMLElement, { adjustFontSize, scale }?: AppendHtmlOptions): void;
    getBlocksInfo(): void;
}
export {};
