var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import { createWorker } from 'tesseract.js';
var Ocr = /** @class */ (function () {
    function Ocr(_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.language, language = _c === void 0 ? 'eng' : _c, _d = _b.lineClassName, lineClassName = _d === void 0 ? '' : _d, _e = _b.wordClassName, wordClassName = _e === void 0 ? '' : _e;
        this.data = null;
        this.language = null;
        this.lineClassName = '';
        this.wordClassName = '';
        this.progressListeners = [];
        this.minFontSize = 2;
        this.language = language;
        this.lineClassName = lineClassName;
        this.wordClassName = wordClassName;
    }
    Ocr.prototype.addProgressListener = function (listener) {
        this.progressListeners.push(listener);
    };
    Ocr.prototype.removeProgressListener = function (listener) {
        this.progressListeners = this.progressListeners.filter(function (item) { return item !== listener; });
    };
    /**
     * Parse image with tesseract
     *
     * @param {ImageLike} image The image instance.
     */
    Ocr.prototype.parseImage = function (image) {
        return __awaiter(this, void 0, void 0, function () {
            var worker, data;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        worker = createWorker({
                            logger: function (m) {
                                _this.progressListeners.forEach(function (callback) {
                                    callback(m);
                                });
                            },
                        });
                        return [4 /*yield*/, worker.load()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, worker.loadLanguage(this.language)];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, worker.initialize(this.language)];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, worker.setParameters({
                                tessedit_pageseg_mode: "2" /* PSM.AUTO_ONLY */,
                            })];
                    case 4:
                        _a.sent();
                        return [4 /*yield*/, worker.recognize(image)];
                    case 5:
                        data = (_a.sent()).data;
                        this.data = data;
                        return [4 /*yield*/, worker.terminate()];
                    case 6:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    Ocr.prototype.updateTextSize = function (el) {
        var wrap = document.createElement('span');
        document.body.appendChild(wrap);
        wrap.innerHTML = el.textContent.trim();
        var height = parseInt(el.style.height.replace('px', ''));
        var width = parseInt(el.style.width.replace('px', ''));
        var fontSize = height;
        if (fontSize > this.minFontSize) {
            wrap.style.fontSize = "".concat(fontSize, "px");
            var fontSizeH = fontSize * height / wrap.offsetHeight;
            var fontSizeW = fontSize * width / wrap.offsetWidth;
            fontSize = fontSizeH < fontSizeW ? fontSizeH : fontSizeW;
        }
        if (fontSize < this.minFontSize) {
            fontSize = this.minFontSize;
        }
        el.style.fontSize = "".concat(fontSize, "px");
        wrap.remove();
    };
    Ocr.prototype.updatePosition = function (el, bbox, scale, parentBbox) {
        var x0 = bbox.x0, x1 = bbox.x1, y0 = bbox.y0, y1 = bbox.y1;
        el.style.position = 'absolute';
        el.style.left = "".concat(parentBbox ? (parentBbox.x0 * scale) : (x0 * scale), "px");
        el.style.top = "".concat(parentBbox ? 0 : (y0 * scale), "px");
        el.style.width = "".concat((x1 - x0) * scale, "px");
        el.style.height = "".concat((y1 - y0) * scale, "px");
    };
    Ocr.prototype.mapWords = function (words, scale, adjustFontSize, parentBbox) {
        var wordElementsList = [];
        for (var i = 0; i < words.length; i++) {
            var _a = words[i], bbox = _a.bbox, text = _a.text;
            var nextBboxX = i < words.length - 1 ? words[i + 1].bbox.x0 : bbox.x1;
            if (!text.trim().length)
                continue;
            var el = document.createElement('span');
            el.innerText = text;
            if (this.wordClassName.length) {
                el.classList.add(this.wordClassName);
            }
            var elementBbox = { x0: bbox.x0, x1: nextBboxX, y0: parentBbox.y0, y1: parentBbox.y1 };
            this.updatePosition(el, elementBbox, scale, {
                x0: bbox.x0 - parentBbox.x0,
                x1: bbox.x1 - parentBbox.x1,
                y0: bbox.y0 - parentBbox.y0,
                y1: bbox.y1 - parentBbox.y1
            });
            if (adjustFontSize) {
                this.updateTextSize(el);
            }
            wordElementsList.push(el);
        }
        return wordElementsList;
    };
    Ocr.prototype.mapLine = function (line, scale, adjustFontSize) {
        var bbox = line.bbox, words = line.words;
        var el = document.createElement('span');
        if (this.lineClassName.length) {
            el.classList.add(this.lineClassName);
        }
        this.updatePosition(el, bbox, scale);
        var wordElements = this.mapWords(words, scale, adjustFontSize, bbox);
        if (!wordElements.length)
            return null;
        wordElements.forEach(function (word) {
            el.appendChild(word);
        });
        return el;
    };
    /**
     * Append generated html to the container.
     *
     * @param {HTMLElement} container The container where elements should append.
     * @param {AppendHtmlOptions} options The options for customization.
     */
    Ocr.prototype.appendHtml = function (container, _a) {
        var _this = this;
        var _b = _a === void 0 ? {} : _a, _c = _b.adjustFontSize, adjustFontSize = _c === void 0 ? true : _c, _d = _b.scale, scale = _d === void 0 ? 1 : _d;
        if (!this.data.hocr)
            return;
        var elements = [];
        this.data.lines.forEach(function (line) {
            var mappedLine = _this.mapLine(line, scale, adjustFontSize);
            if (mappedLine) {
                elements = __spreadArray(__spreadArray([], elements, true), [mappedLine], false);
            }
        });
        elements.forEach(function (element) {
            container.appendChild(element);
        });
    };
    Ocr.prototype.getBlocksInfo = function () {
    };
    return Ocr;
}());
export default Ocr;
